@@ -102,6 +102,40 @@ ImageOutput::~ImageOutput()
 
 
 
+// Utility: Make sure the provided data span is the right size for the
+// image described by spec and datatype. If they don't match, issue an
+// error and return false.
+static bool
+check_span_size(ImageOutput* out, string_view caller, const ImageSpec& spec,
+                TypeDesc datatype, imagesize_t npixels,
+                const image_span<const std::byte>& data)
+{
+    // One of two things must be correct: Either format is Unknown and the
+    // total byte size needs to match the "native" size, or the format is
+    // concrete and the number of value must match (it's ok if the size
+    // doesn't match, since a data type conversion will occur).
+    if (datatype.is_unknown()) {  // Unknown assumes native chan types
+        size_t sz = npixels * spec.pixel_bytes(true);
+        if (sz != data.size_bytes()) {
+            out->errorfmt(
+                "{}: image_span size is incorrect ({} bytes vs {} needed)",
+                caller, data.size_bytes(), sz);
+            return false;
+        }
+    } else {  // single concrete type
+        size_t nvals = npixels * size_t(spec.nchannels);
+        if (nvals != data.nvalues()) {
+            out->errorfmt(
+                "{}: image_span size is incorrect ({} values vs {} needed)",
+                caller, data.nvalues(), nvals);
+            return false;
+        }
+    }
+    return true;
+}
+
+
+
 bool
 ImageOutput::write_scanline(int /*y*/, int /*z*/, TypeDesc /*format*/,
                             const void* /*data*/, stride_t /*xstride*/)
@@ -120,13 +154,9 @@ ImageOutput::write_scanline(int y, TypeDesc format,
         errorfmt("write_scanlines: Invalid scanline index {}", y);
         return false;
     }
-    size_t sz = m_spec.scanline_bytes(format);
-    if (sz != data.size_bytes()) {
-        errorfmt(
-            "write_scanline: Buffer size is incorrect ({} bytes vs {} needed)",
-            sz, data.size_bytes());
+    if (!check_span_size(this, "write_scanline", m_spec, format, m_spec.width,
+                         data))
         return false;
-    }
 
     // Default implementation (for now): call the old pointer+stride
     return write_scanline(y, 0, format, data.data(), data.xstride());
@@ -164,13 +194,9 @@ ImageOutput::write_scanlines(int ybegin, int yend, TypeDesc format,
         errorfmt("write_scanlines: Invalid scanline range {}-{}", ybegin, yend);
         return false;
     }
-    size_t sz = m_spec.scanline_bytes(format) * size_t(yend - ybegin);
-    if (sz != data.size_bytes()) {
-        errorfmt(
-            "write_scanlines: Buffer size is incorrect ({} bytes vs {} needed)",
-            sz, data.size_bytes());
+    if (!check_span_size(this, "write_scanlines", m_spec, format,
+                         m_spec.width * size_t(yend - ybegin), data))
         return false;
-    }
 
     // Default implementation (for now): call the old pointer+stride
     return write_scanlines(ybegin, yend, 0, format, data.data(), data.xstride(),
@@ -194,15 +220,9 @@ bool
 ImageOutput::write_tile(int x, int y, int z, TypeDesc format,
                         const image_span<const std::byte>& data)
 {
-    size_t sz = format == TypeUnknown
-                    ? m_spec.pixel_bytes(true /*native*/)
-                    : m_spec.tile_pixels() * size_t(m_spec.nchannels)
-                          * format.size();
-    if (sz != data.size_bytes()) {
-        errorfmt("write_tile: Buffer size is incorrect ({} bytes vs {} needed)",
-                 sz, data.size_bytes());
+    if (!check_span_size(this, "write_tile", m_spec, format,
+                         m_spec.tile_pixels(), data))
         return false;
-    }
 
     // Default implementation (for now): call the old pointer+stride
     return write_tile(x, y, z, format, data.data(), data.xstride(),
@@ -691,12 +711,9 @@ bool
 ImageOutput::write_image(TypeDesc format,
                          const image_span<const std::byte>& data)
 {
-    size_t sz = m_spec.image_bytes(/*native=*/format == TypeUnknown);
-    if (sz != data.size_bytes()) {
-        errorfmt("write_image: Buffer size is incorrect ({} bytes vs {} needed)",
-                 sz, data.size_bytes());
+    if (!check_span_size(this, "write_image", m_spec, format,
+                         m_spec.image_pixels(), data))
         return false;
-    }
 
     // Default implementation (for now): call the old pointer+stride
     return write_image(format, data.data(), data.xstride(), data.ystride(),
